@@ -1,12 +1,15 @@
+import json
 import os
 import shutil
-from collections import OrderedDict
+from datetime import timezone
 from functools import wraps
+from hashlib import md5
 from os import path
 from signal import getsignal
 from signal import SIGINT
 from signal import signal
 
+import dateutil.parser
 from click import echo
 from click import get_current_context
 from click import style
@@ -30,45 +33,47 @@ def get_user_project_path(*subdirs):
     """The path on your user dir to /.code42cli/[subdir]."""
     package_name = __name__.split(".")[0]
     home = path.expanduser("~")
-    hidden_package_name = ".{}".format(package_name)
+    hidden_package_name = f".{package_name}"
     user_project_path = path.join(home, hidden_package_name)
     result_path = path.join(user_project_path, *subdirs)
     if not path.exists(result_path):
         os.makedirs(result_path)
+
     return result_path
 
 
-def find_format_width(record, header, include_header=True):
+def find_format_width(records, header, include_header=True):
     """Fetches needed keys/items to be displayed based on header keys.
 
     Finds the largest string against each column so as to decide the padding size for the column.
 
     Args:
-        record (dict): data to be formatted.
-        header (dict): key-value where keys should map to keys of record dict and
+        records (list or dict): A list of data to be formatted.
+        header (dict): Key-value where keys should map to keys of record dict and
           value is the corresponding column name to be displayed on the CLI.
-        include_header (bool): include header in output, defaults to True.
+        include_header (bool): Include header in output, defaults to True.
 
     Returns:
         tuple (list of dict, dict): i.e Filtered records, padding size of columns.
     """
+    if isinstance(records, dict):
+        records = [records]
+
     rows = []
     if include_header:
         if not header:
-            header = _get_default_header(record)
+            header = _get_default_header(records)
         rows.append(header)
-    max_width_item = dict(header.items())  # Copy
-    for record_row in record:
-        row = OrderedDict()
+    widths = dict(header.items())  # Copy
+    for record_row in records:
+        row = {}
         for header_key in header.keys():
             item = record_row.get(header_key)
             row[header_key] = item
-            max_width_item[header_key] = max(
-                max_width_item[header_key], str(item), key=len
-            )
+            widths[header_key] = max(widths[header_key], str(item), key=len)
         rows.append(row)
-    column_size = {key: len(value) for key, value in max_width_item.items()}
-    return rows, column_size
+    column_sizes = {key: len(value) for key, value in widths.items()}
+    return rows, column_sizes
 
 
 def format_to_table(rows, column_size):
@@ -90,7 +95,7 @@ def format_string_list_to_columns(string_list, max_width=None):
         max_width, _ = shutil.get_terminal_size()
     column_width = len(max(string_list, key=len)) + _PADDING_SIZE
     num_columns = int(max_width / column_width) or 1
-    format_string = "{{:<{0}}}".format(column_width) * num_columns
+    format_string = f"{{:<{column_width}}}" * num_columns
     batches = [
         string_list[i : i + num_columns]
         for i in range(0, len(string_list), num_columns)
@@ -134,7 +139,7 @@ class warn_interrupt:
     def _handle_interrupts(self, sig, frame):
         if not self.interrupted:
             self.interrupted = True
-            echo("\n{}\n{}".format(self.warning, self.exit_instructions), err=True)
+            echo(f"\n{self.warning}\n{self.exit_instructions}", err=True)
         else:
             exit()
 
@@ -152,6 +157,7 @@ def get_url_parts(url_str):
     port = None
     if len(parts) > 1 and parts[1] != "":
         port = int(parts[1])
+
     return parts[0], port
 
 
@@ -166,4 +172,29 @@ def _get_default_header(header_items):
         for key in keys:
             if key not in header and isinstance(key, str):
                 header[key] = key
+
     return header
+
+
+def hash_event(event):
+    if isinstance(event, dict):
+        event = json.dumps(event, sort_keys=True)
+    return md5(event.encode()).hexdigest()
+
+
+def print_numbered_list(items):
+    """Outputs a numbered list of items to the user.
+    For example, provide ["test", "foo"] to print "1. test\n2. foo".
+    """
+
+    choices = dict(enumerate(items, 1))
+    for num in choices:
+        echo(f"{num}. {choices[num]}")
+    echo()
+
+
+def parse_timestamp(date_str):
+    # example: {"property": "bar", "timestamp": "2020-11-23T17:13:26.239647Z"}
+    ts = date_str[:-1]
+    date = dateutil.parser.parse(ts).replace(tzinfo=timezone.utc)
+    return date.timestamp()
